/*
 * 
 * deltaTimer 0.01 -- Simple Pocket PC Countdown Timer
 * 
 * Copyright (C) 2005 Christoph Sommer 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * deltaTimer's homepage is at
 *     http://www.deltadevelopment.de/users/christoph/deltaTimer/
 * 
 */

using System;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.Data;
using System.IO;
using System.Runtime.InteropServices;


public class DeltaTimer : System.Windows.Forms.Form {
	[DllImport("coredll.dll")] protected static extern void SystemIdleTimerReset();
	[DllImport("coredll.dll")] protected static extern int PlaySound(string szSound, IntPtr hMod, int flags);

	protected static Pen blackPen = new Pen(Color.Black);

	protected int secondsToGo = 0;
	protected Image clockImage;
	protected bool mouseIsDown = false;

	private System.Windows.Forms.Button btnGo;
	private System.Windows.Forms.MenuItem mAbout;
	private System.Windows.Forms.MenuItem mExit;
	private System.Windows.Forms.Timer tTick;
	private System.Windows.Forms.MainMenu mMain;
	private System.Windows.Forms.Label lClock;
	private System.Windows.Forms.PictureBox pbClock;
	private System.Windows.Forms.ListBox lbAbout;

	/**
	 * Called when counter reaches zero
	 */
	protected void notifyUser() {
		PlaySound("\\Windows\\Alarm1", IntPtr.Zero, 0x20001);
	}

	/**
	 * Creates a ready-made bitmap of a plain clock for quicker updates
	 */
	protected static Image createClockImage(int width) {
		Image image = new Bitmap(width, width);
		Graphics canvas = Graphics.FromImage(image);

		int center = width/2;
		int clockr = width/2;

		canvas.Clear(Color.White);
		canvas.DrawEllipse(blackPen, center-clockr, center-clockr, 2*clockr, 2*clockr);
		for (int i = 0; i < 60; i++) {
			double beta = (2*Math.PI)/60*(i - 15);
			double l1 = (double)clockr * (i%5==0?0.80:0.90);
			double l2 = (double)clockr * 0.95;
			int y1 = (int)Math.Round(l1 * Math.Sin(beta));
			int x1 = (int)Math.Round(l1 * Math.Cos(beta));
			int y2 = (int)Math.Round(l2 * Math.Sin(beta));
			int x2 = (int)Math.Round(l2 * Math.Cos(beta));
			canvas.DrawLine(blackPen, center+x1, center+y1, center+x2, center+y2);
		}

		return image;
	}

	/**
	 * Calculates where (in terms of seconds) on the clock the user clicked
	 */
	protected int mousePosToSeconds(int x, int y) {
		int canvaswidth = pbClock.Width;
		int center = canvaswidth/2;
		double angle = Math.Atan2(y - center, x - center);

		return ((int)Math.Round((60*60)/(2*Math.PI)*angle) + 60*(15 + 60)) % (60*60);
	}

	/**
	 * Copies the clock image to the canvas and adds a minute hand
	 */
	protected void drawClock(Graphics canvas, int canvaswidth, int seconds) {
		int center = canvaswidth/2;
		int clockr = (canvaswidth/2);

		double alpha = (2*Math.PI)/60*((double)seconds/60 - 15);
		double diamondAngle = (2*Math.PI)/360*30;
		double diamondLeg = clockr*0.25;
		double handLength = clockr-5;
		int dy1 = (int)Math.Round(diamondLeg * Math.Sin(alpha+diamondAngle));
		int dx1 = (int)Math.Round(diamondLeg * Math.Cos(alpha+diamondAngle));
		int dy2 = (int)Math.Round(diamondLeg * Math.Sin(alpha-diamondAngle));
		int dx2 = (int)Math.Round(diamondLeg * Math.Cos(alpha-diamondAngle));
		int y = (int)Math.Round(handLength * Math.Sin(alpha));
		int x = (int)Math.Round(handLength * Math.Cos(alpha));

		canvas.DrawImage(clockImage, 0, 0);
		canvas.DrawLine(blackPen, center, center, center+dx1, center+dy1);
		canvas.DrawLine(blackPen, center, center, center+dx2, center+dy2);
		canvas.DrawLine(blackPen, center+dx1, center+dy1, center+x, center+y);
		canvas.DrawLine(blackPen, center+dx2, center+dy2, center+x, center+y);
	}

	/**
	 * Coarse Math.Floor()
	 */
	protected int arbitraryFloor(int number, int steps) {
		int i = number % steps;
		number = number - i;
		return number;
	}
	
	/**
	 * Coarse Math.Round()
	 */
	protected int arbitraryRound(int number, int steps) {
		int i = number % steps;
		number = number - i;
		if (i > steps/2) number+=steps;
		return number;
	}

	/**
	 * Refreshes the user interface
	 */
	protected void updateClock() {
		int hrs = (secondsToGo / 3600);
		int mns = (secondsToGo % 3600) / 60;
		int scs = (secondsToGo % 3600) % 60;
		lClock.Text = (hrs<10?"0":"")+hrs.ToString()+":"+(mns<10?"0":"")+mns.ToString()+":"+(scs<10?"0":"")+scs.ToString();

		this.btnGo.Text = tTick.Enabled?"Stop":"Start";

		int w = pbClock.Width;
		drawClock(Graphics.FromImage(pbClock.Image), w, secondsToGo);
		pbClock.Refresh();
	}

	public DeltaTimer() {
		InitializeComponent();
		pbClock.MouseDown += new System.Windows.Forms.MouseEventHandler(ClockMouseDown);
		pbClock.MouseMove += new System.Windows.Forms.MouseEventHandler(ClockMouseMove);
		pbClock.MouseUp += new System.Windows.Forms.MouseEventHandler(ClockMouseUp);
		pbClock.Image = new Bitmap(pbClock.Width, pbClock.Width);
		clockImage = createClockImage(pbClock.Width);
		updateClock();
	}

	protected override void Dispose(bool disposing) {
		base.Dispose(disposing);
	}

	#region Windows Form Designer generated code
	/// <summary>
	/// Required method for Designer support - do not modify
	/// the contents of this method with the code editor.
	/// </summary>
	private void InitializeComponent()
	{
		this.tTick = new System.Windows.Forms.Timer();
		this.lClock = new System.Windows.Forms.Label();
		this.btnGo = new System.Windows.Forms.Button();
		this.pbClock = new System.Windows.Forms.PictureBox();
		this.mMain = new System.Windows.Forms.MainMenu();
		this.mAbout = new System.Windows.Forms.MenuItem();
		this.mExit = new System.Windows.Forms.MenuItem();
		this.lbAbout = new System.Windows.Forms.ListBox();
		// 
		// tTick
		// 
		this.tTick.Interval = 1000;
		this.tTick.Tick += new System.EventHandler(this.tTick_Tick);
		// 
		// lClock
		// 
		this.lClock.Font = new System.Drawing.Font("Microsoft Sans Serif", 32F, System.Drawing.FontStyle.Regular);
		this.lClock.Location = new System.Drawing.Point(16, 8);
		this.lClock.Size = new System.Drawing.Size(208, 56);
		this.lClock.Text = "00:00:00";
		this.lClock.TextAlign = System.Drawing.ContentAlignment.TopCenter;
		// 
		// btnGo
		// 
		this.btnGo.Location = new System.Drawing.Point(80, 232);
		this.btnGo.Size = new System.Drawing.Size(80, 20);
		this.btnGo.Text = "Start";
		this.btnGo.Click += new System.EventHandler(this.btnGo_Click);
		// 
		// pbClock
		// 
		this.pbClock.Location = new System.Drawing.Point(48, 72);
		this.pbClock.Size = new System.Drawing.Size(145, 145);
		// 
		// mMain
		// 
		this.mMain.MenuItems.Add(this.mAbout);
		this.mMain.MenuItems.Add(this.mExit);
		// 
		// mAbout
		// 
		this.mAbout.Text = "About";
		this.mAbout.Click += new System.EventHandler(this.mAbout_Click);
		// 
		// mExit
		// 
		this.mExit.Text = "Exit";
		this.mExit.Click += new System.EventHandler(this.mExit_Click);
		// 
		// lbAbout
		// 
		this.lbAbout.Enabled = false;
		this.lbAbout.Items.Add("deltaTimer 0.01");
		this.lbAbout.Items.Add("Copyright  2005 Christoph Sommer");
		this.lbAbout.Items.Add("");
		this.lbAbout.Items.Add("http://www.deltadevelopment.de");
		this.lbAbout.Items.Add("        /users/christoph/deltaTimer/");
		this.lbAbout.Items.Add("");
		this.lbAbout.Items.Add("Plays \"\\Windows\\Alarm1\"");
		this.lbAbout.Items.Add("when counter reaches zero.");
		this.lbAbout.Items.Add("Click and drag on the clock ");
		this.lbAbout.Items.Add("or use the dpad to set the time.");
		this.lbAbout.Items.Add("");
		this.lbAbout.Items.Add("I\'d love to get some feedback!");
		this.lbAbout.Location = new System.Drawing.Point(8, 72);
		this.lbAbout.Size = new System.Drawing.Size(224, 184);
		this.lbAbout.Visible = false;
		// 
		// DeltaTimer
		// 
		this.Controls.Add(this.lbAbout);
		this.Controls.Add(this.pbClock);
		this.Controls.Add(this.btnGo);
		this.Controls.Add(this.lClock);
		this.Menu = this.mMain;
		this.Text = "deltaTimer";
		this.KeyDown += new System.Windows.Forms.KeyEventHandler(this.Form1_KeyDown);

	}
	#endregion

	static void Main() 
	{
		Application.Run(new DeltaTimer());
	}

	private void btnGo_Click(object sender, System.EventArgs e) {
		tTick.Enabled = !tTick.Enabled;
		updateClock();
		this.Focus();
	}

	/**
	 * Called every second
	 */
	private void tTick_Tick(object sender, System.EventArgs e) {
		SystemIdleTimerReset();

		secondsToGo--; 
		if (secondsToGo <= 0) {
			tTick.Enabled = false;
			secondsToGo = 0;
			notifyUser();
		}
		updateClock();			
	}

	protected void moveHand(int x, int y) {
		int secs = arbitraryRound(mousePosToSeconds(x, y), 60);

		if ((secs > 55*60) && (secondsToGo % (60*60)) < 5*60) {
			secs = arbitraryFloor(secondsToGo, 60*60) + secs - 60*60;
		} 
		else if ((secs < 5*60) && (secondsToGo % (60*60)) > 55*60) {
			secs = arbitraryFloor(secondsToGo, 60*60) + secs + 60*60;
		} else {
			secs = arbitraryFloor(secondsToGo, 60*60) + secs;
		}

		if (secs < 0) secs = 0;
		if (secs > 86400) secs = 86400;

		secondsToGo = secs;
		updateClock();		
	}

	private void ClockMouseDown(object sender, System.Windows.Forms.MouseEventArgs e) {
		if (tTick.Enabled) return;
		mouseIsDown = true;

		moveHand(e.X, e.Y);
	}

	private void ClockMouseMove(object sender, System.Windows.Forms.MouseEventArgs e) {
		if (!mouseIsDown) return;

		moveHand(e.X, e.Y);
	}

	private void ClockMouseUp(object sender, System.Windows.Forms.MouseEventArgs e) {
		mouseIsDown = false;
	}

	private void Form1_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e) {
		if (e.KeyCode == Keys.Return) {
			btnGo_Click(sender, e);
			return;
		}
		if (tTick.Enabled) return;

		int secs = secondsToGo;

		if (e.KeyCode == Keys.Up) secs = arbitraryRound(secs, 60) + 60;
		if (e.KeyCode == Keys.Down) secs = arbitraryRound(secs, 60) - 60;
		if (e.KeyCode == Keys.Right) secs = secs + 1;
		if (e.KeyCode == Keys.Left) secs = secs - 1;

		if (secs < 0) secs = 0;
		if (secs > 86400) secs = 86400;

		secondsToGo = secs;
		updateClock();
	}

	private void mExit_Click(object sender, System.EventArgs e) {
		Close();
	}

	private void mAbout_Click(object sender, System.EventArgs e) {
		pbClock.Visible = !pbClock.Visible;
		lbAbout.Visible = !lbAbout.Visible;
	}

}
